// depends on "itemdata.cs"

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

datablock ItemData( PuddleData ){
  category = "Puddle";

  // Next, we'll tell the mission editor where to find the .dts shape file 
  shapeFile = "~/data/shapes/puddle/puddle.dts";
  
  collisionLimitX = 1.5;
  collisionLimitY = 1.5;
  collisionLimitZ = 1;
  
  // group that contains all puddles
  puddleGroupName = "MissionGroup/PuddleGroup";
  
  puddleDamageOn = true;
};

//-----------------------------------------------------------------------------

function PuddleData::bulletHit(%this, %obj){
  %obj.health -= 40;
  
  
  // if we need to delete this puddle
  if (%obj.health < 0){
    
    // we need to find associated particle emitter
    %groupName = PuddleData.puddleGroupName;
    %group = nameToID(%groupName);
    if (%group != -1) {
      %count = %group.getCount();
      // for each object in group
      for (%i = 0; %i < %count; %i++){
        %thisob = %group.getObject(%i);
        
        // if this is a particle emitter
        if (%thisob.getClassName() $= "ParticleEmitterNode"){
          // if this is the one we need to delete
          if ((%thisob.autoGeneratedTag $= "***PUDDLEPARTICLEOBJECT***") && (%thisob.emitterID == %obj.emitterID)){
            %thisob.delete();
            break;
          }
        }// not a particle emitter
        
      } // for each item in puddle group
      
    // if missing puddles group
    }else{
      error("Missing puddle group " @ %groupName);
      return;
    }
    // delete the puddle now    
    %obj.delete();    
  }
}

//-----------------------------------------------------------------------------

function PuddleData::getCollisionBox(%this, %obj){
  %thisCenter = %obj.getWorldBoxCenter();
  
  %xcom = getWord(%thisCenter, 0);
  %ycom = getWord(%thisCenter, 1);
  %zcom = getWord(%thisCenter, 2);
  
  %minx = %xcom - %this.collisionLimitX;
  %miny = %ycom - %this.collisionLimitY;
  %minz = %zcom - %this.collisionLimitZ;
  
  %maxx = %xcom + %this.collisionLimitX;
  %maxy = %ycom + %this.collisionLimitY;
  %maxz = %zcom + %this.collisionLimitZ;
  
  return (%minx SPC %miny SPC %minz SPC %maxx SPC %maxy SPC %maxz);
}

//-----------------------------------------------------------------------------

function PuddleData::isPointColliding(%this, %obj, %pt){
  %thisCenter = %obj.getWorldBoxCenter();
  
  %xcom = getWord(%thisCenter, 0);
  %ycom = getWord(%thisCenter, 1);
  %zcom = getWord(%thisCenter, 2);
  
  %minx = %xcom - %this.collisionLimitX;
  %miny = %ycom - %this.collisionLimitY;
  %minz = %zcom - %this.collisionLimitZ;
  
  %maxx = %xcom + %this.collisionLimitX;
  %maxy = %ycom + %this.collisionLimitY;
  %maxz = %zcom + %this.collisionLimitZ;
  
  %ptx = getWord(%pt,0);
  %pty = getWord(%pt,1);
  %ptz = getWord(%pt,2);

  // if NOT inside 
  if (((%ptx > %maxx) || (%pty > %maxy) || (%ptz > %maxz)) || 
      ((%ptx < %minx) || (%pty < %miny) || (%ptz < %minz))){
  
    return false;
  // else we must be colliding
  }else{
    return true;
  }
}

//-----------------------------------------------------------------------------

function PuddleData::onCollision( %this, %obj, %col ){
  if (%col.getClassName() $= "Player"){
    // when we take damage, we turn all puddle damage "off" for a bit of time to ensure that overlapping puddles don't do more damage than should be done
    if (%this.puddleDamageOn == true){
      %this.puddleDamageOn = false;
      %col.damage("", "0 0 0", 37, "Insecticide Puddle"); 
      %this.schedule(700, "damageOn");
    }
  }
}

//-----------------------------------------------------------------------------

function PuddleData::damageOn(%this){
  %this.puddleDamageOn = true;
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

datablock ParticleData( PuddleParticle ){
	dragCoefficient    = 0.6;
  windCoefficient    = 0.0;
  gravityCoefficient = 0.01; // Gravity pulls down, but a negative value makes the particles rise instead of sink
	inheritedVelFactor = 1.00;
	constantAcceleration = 0.0;
	lifetimeMS         = 6500;
	lifetimeVarianceMS = 1000;
  //spinSpeed = 
	spinRandomMin      = -30.0;
	spinRandomMax      = 30.0;
	useInvAlpha        = false;
  animateTexture    = false;  
  //framesPerSec = 
  textureName        = "~/data/shapes/particletextures/vatglow";
  //animTexName[0] =

  //
	// Color does not vary over time, but 
	// It becomes more and more transparent over time (no pop)
	//
	//colors[0] = "0.0 1.0 0.0 0.1";
	//colors[1] = "0.0 1.0 0.0 0.075";
	//colors[2] = "0.0 1.0 0.0 0.05";
	//colors[3] = "0.0 1.0 0.0 0.25";
  colors[0] = "0.0 1.0 0.0 0.2";
	colors[1] = "0.0 1.0 0.0 0.15";
	colors[2] = "0.0 1.0 0.0 0.10";
	colors[3] = "0.0 1.0 0.0 0.05";
	colors[4] = "0.0 1.0 0.0 0.01";

  //
	// Sizes do not vary over time
	//
	sizes[0] = 0.5;
	sizes[1] = 0.5;
	sizes[2] = 0.5;
  sizes[3] = 0.5;
  sizes[4] = 0.5;
  
	//times[0] = 0.0;
	//times[1] = 2.0;
	//times[2] = 4.0;
  //times[3] = 6.0;
  //times[4] = 1000.0;
};

datablock ParticleEmitterData( PuddleParticleEmitter ){
  ejectionPeriodMS = 200;
  periodVarianceMS = 50;
  ejectionVelocity = 1.0;//0.04;
  velocityVariance = 0.5;//1;
  ejectionOffset = 0;
  thetaMin = 0.0;
  thetaMax = 180;
  phiReferenceVel = 360;
  phiVariance = 180;
  //overrideAdvance =
  //orientParticles =
  //orientOnVelocity =
	particles = "PuddleParticle";
  //lifetimeMS =
  //lifetimeVariance = 
};

datablock ParticleEmitterNodeData( PuddleParticleEmitterNode ){
	timeMultiple = 1;
};

