
// deletes and recreates any objects that were automatically generated
function updateEffects(%isorange){
  updateEffects_Recursive(MissionGroup.getId(), true, %isorange);
}

// removes any objects that were automatically generated
function deleteEffects(){
  updateEffects_Recursive(MissionGroup.getId(), false);
}

// removes any objects that were automatically generated
function removeEffects(){
  // just lets the user use removeEffects() instead of deleteEffects()
  deleteEffects();
}

// recursive function that updates auto-generated objects
function updateEffects_Recursive(%obj, %regenerate, %isorange){
  %puddleEmitterID = 0;
  
  if (%isOrange $= ""){
    %isOrange = false;
  }
  
  // first, trash any objects that were auto-generated
  %i = 0;
  // for each object in this group
  while (%i < %obj.getCount()){
    // get the current child object
    %thisone = %obj.getObject(%i);
    
    // get it's class name
    %classname = %thisone.getClassName();
    
    // if this child is a particle emitter
    if (%classname $= "ParticleEmitterNode"){ 
      // AND it has the proper tag that identifies it as one that was automatically created
      if (%thisone.autoGeneratedTag $= "***VATPARTICLEOBJECT***"){
        echo("-->Deleting old vat particle emitter.");
        
        // delete the object
        %thisone.delete();
        
        // start over at the beginning of this group, because our deletion may have re-ordered things
        %i = -1;
      }else if(%thisone.autoGeneratedTag $= "***PUDDLEPARTICLEOBJECT***"){
        echo("-->Deleting puddle particle emitter.");
        
        // delete the object
        %thisone.delete();
        
        // start over at the beginning of this group, because our deletion may have re-ordered things
        %i = -1;
      }
      
    // if this is a trigger
    }else if (%classname $= "Trigger"){ 
      // AND it has the proper tag that identifies it as one that was automatically created
      if (%thisone.autoGeneratedTag $= "***VATTRIGGEROBJECT***"){
        echo("-->Deleting old vat trigger.");
        
        // delete the object
        %thisone.delete();
        
        // start over at the beginning of this group, because our deletion may have re-ordered things
        %i = -1;
      }
    }
    
    // next child
    %i++;
  }
  // at this point, any automatically generated objects in this group have been deleted
  
  
  // loop for each object in this group
  for (%i = 0; %i < %obj.getCount(); %i++){
    %thisone = %obj.getObject(%i);
    
    %classname = %thisone.getClassName();
    
    
    
    // if this is a group, we need to recursively perform the same routine on that group
    if (%classname $= "SimGroup"){
      // recursively process SimGroups
      updateEffects_Recursive(%thisone, %regenerate, %isorange);
      
      
    }else{
      // if we should re-create the auto-generated objects
      if (%regenerate == true){
        
        
        
        // if this is an interior instance
        if (%classname $= "InteriorInstance"){
          
          // if this is a vat type 1
          if (%thisone.interiorFile $= "RAIDOnXanthuVII/data/interiors/common/vat1.dif"){
            echo("-->Processing Vat Type 1:");
            
            echo("---->Adding vat particle emitter.");
            %emitName = "Vat1ParticleEmitter";
            if (%isOrange == true){
              %emitName = %emitName @ "Poisoned";
            }
            // create the particle emitter
            %elFogo = new ParticleEmitterNode(){
              canSaveDynamicFields = "1";
              position = VectorAdd(%thisone.getWorldBoxCenter(), "0 0 -1.0");
              rotation = "1 0 0 0";
              scale = "1 1 1";
              dataBlock = "Vat1ParticleEmitterNode";
              emitter = %emitName;
              velocity = "1";
            };
            // tag this particle emitter object
            %elFogo.autoGeneratedTag = "***VATPARTICLEOBJECT***";
            // add the object to the group
            %obj.add(%elFogo);
            
            
            
            echo("---->Adding vat trigger.");
            %trigScale = "6 6 6";
            %trigPosition = VectorScale(%trigScale, 0.5);
            %trigPosition = (getWord(%trigPosition, 0)*-1.0) SPC getWord(%trigPosition, 1) SPC (getWord(%trigPosition, 2) * -1.0);
            %trigPosition = VectorAdd(%thisone.getWorldBoxCenter(), %trigPosition);
            %trigPosition = VectorAdd(%trigPosition, "0 0 0.75");
            // create the particle emitter
            %trig = new Trigger(){
              canSaveDynamicFields = "1";
              position = %trigPosition;
              rotation = "1 0 0 0";
              scale = %trigScale;
              dataBlock = "VatTrigger";
              polyhedron = "0 0 0 1 0 0 0 -1 0 0 0 1";
              
              // dynamic fields
              autoGeneratedTag = "***VATTRIGGEROBJECT***";
              damageRate = 40;
            };
            // add the object to the group
            %obj.add(%trig);
            
            
            
            echo("-->End Processing Vat Type 1.");
          } // endif this is vat type 1
          
          
          
          // if this is vat type 2
          if (%thisone.interiorFile $= "RAIDOnXanthuVII/data/interiors/common/vat2.dif"){
            echo("-->Processing Vat Type 2:");
            
            echo("---->Adding vat particle emitter.");
            %emitName = "Vat2ParticleEmitter";
            if (%isOrange == true){
              %emitName = %emitName @ "Poisoned";
            }
            // create the particle emitter
            %elFogo = new ParticleEmitterNode(){
              canSaveDynamicFields = "1";
              position = VectorAdd(%thisone.getWorldBoxCenter(), "0 0 0");
              rotation = "1 0 0 0";
              scale = "1 1 1";
              dataBlock = "Vat2ParticleEmitterNode";
              emitter = %emitName;
              velocity = "1";
            };
            // tag this fog object
            %elFogo.autoGeneratedTag = "***VATPARTICLEOBJECT***";
            // add the object to the group
            %obj.add(%elFogo);
            
            
            
            echo("---->Adding vat trigger.");
            // create the particle emitter
            %trigScale = "3.5 3.5 1.0";
            %trigPosition = VectorScale(%trigScale, 0.5);
            %trigPosition = (getWord(%trigPosition, 0)*-1.0) SPC getWord(%trigPosition, 1) SPC (getWord(%trigPosition, 2) * -1.0);
            %trigPosition = VectorAdd(%thisone.getWorldBoxCenter(), %trigPosition);
            %trigPosition = VectorAdd(%trigPosition, "0 0 0.75");
            %trig = new Trigger(){
              canSaveDynamicFields = "1";
              scale = %trigScale;
              position = %trigPosition;
              rotation = "1 0 0 0";
              dataBlock = "VatTrigger";
              polyhedron = "0 0 0 1 0 0 0 -1 0 0 0 1";
              
              // dynamic fields
              autoGeneratedTag = "***VATTRIGGEROBJECT***";
              damageRate = 40;
            };
            // add the object to the group
            %obj.add(%trig);
            
            
            
            echo("-->End Processing Vat Type 2.");
          } //endif this is vat type 2
          
          
        } //endif this is an interior instance object
        
        
        // if this is an item
        else if(%classname $= "Item"){
          
          // if this is a puddle and we are not currently "orange" 
          // see, we will be emitting gas particles if we are "orange" so we want to avoid too many particle effects at once
          if ((%thisone.isAPuddle $= "true") && (%isOrange == false)){
            echo("-->Processing Insecticide Puddle:");
            
            
            
            echo("---->Adding particle emitter.");
            %trigScale = "1 1 1";          
            %trigPosition = %thisone.getWorldBoxCenter();
            %trigPosition = VectorAdd(%trigPosition, "0 0 -0.25");
            %thisone.emitterID = %puddleEmitterID;
            
            // create the particle emitter
            %elFogo = new ParticleEmitterNode(){
              canSaveDynamicFields = "1";
              position = %trigPosition;
              rotation = "1 0 0 0";
              scale = %trigScale;
              dataBlock = "PuddleParticleEmitterNode";
              emitter = "PuddleParticleEmitter";
              velocity = "1";
              emitterID = %puddleEmitterID;
            };
            
            %puddleEmitterID++;
            // tag this particle emitter object
            %elFogo.autoGeneratedTag = "***PUDDLEPARTICLEOBJECT***";
            // add the object to the group
            %obj.add(%elFogo);
            
            
            
            echo("-->Done Processing Insecticide Puddle.");
          } // endif this is a puddle
          
        } //endif this is an Item
        
      
      
      }  //endif (%regenerate == true)
    }
  }  // endfor each child object
  return;
}
