//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function onServerCreated(){
  // Server::GameType is sent to the master server.
  // This variable should uniquely identify your game and/or mod.
  $Server::GameType = "Defender2";

  // Server::MissionType sent to the master server.  Clients can
  // filter servers based on mission type.
  $Server::MissionType = "None";

  // GameStartTime is the sim time the game started. Used to calculated
  // game elapsed time.
  $Game::StartTime = 0;

  // Load up all datablocks, objects etc.  This function is called when a server is constructed.
  exec("./gameConnection.cs");
  exec("./audioProfiles.cs");
  exec("./storylineHandler.cs");
  exec("./camera.cs");
  exec("./commands.cs");
  exec("./centerprint.cs");
  exec("./markers.cs"); 
  exec("./triggers.cs"); 
  exec("./inventory.cs");
  exec("./shapeBase.cs");
  exec("./staticShape.cs");
  exec("./itemdata.cs");
  exec("./neutralizeritem.cs");
  exec("./agentItem.cs");
  exec("./puddle.cs");

  exec("common/server/lightingSystem.cs");

  exec("./neutralizerWeapon.cs");
  exec("./mindWeapon.cs");
  exec("./pistolWeapon.cs");
  
  exec("./player.cs");
  exec("./aiplayer.cs");
  exec("./aimanager.cs");
  exec("./allyPlayer.cs");
  exec("./soldier.cs");
  
  exec("./checkpointTrigger.cs");
  exec("./masterVatTrigger.cs");
  exec("./AntiRetardTrigger.cs");
  exec("./FinalTrigger.cs");
  exec("./Endsequencetrigger.cs");

  exec("./vatFx.cs");
  exec("./vatTriggers.cs");

  exec("./editorhelper.cs");

  // Keep track of when the game started
  $Game::StartTime = $Sim::Time;
}

//-----------------------------------------------------------------------------

function onServerDestroyed(){
   // This function is called as part of a server shutdown.
}

//-----------------------------------------------------------------------------

function onMissionLoaded(){
  // Called by loadMission() once the mission is finished loading.
  // Nothing special for now, just start up the game play.
  startGame();
}

//-----------------------------------------------------------------------------

function onMissionEnded(){
  // Called by endMission(), right before the mission is destroyed

  // Normally the game should be ended first before the next
  // mission is loaded, this is here in case loadMission has been
  // called directly.  The mission will be ended if the server
  // is destroyed, so we only need to cleanup here.
  cancel($Game::Schedule);
  $Game::Running = false;
}

//-----------------------------------------------------------------------------

function startGame(){
  if ($Game::Running) {
    error("startGame: End the game first!");
    return;
  }

  // Inform the client we're starting up
  for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
    %cl = ClientGroup.getObject( %clientIndex );
    commandToClient(%cl, 'GameStart');

    // Other client specific setup..
    %cl.score = 0;
  }

  $Game::Running = true;
   
  // Start the AIManager
  new ScriptObject(AIManager) {};
  MissionCleanup.add(AIManager);
  AIManager.initialize();
  AIManager.think();
}

//-----------------------------------------------------------------------------

function endGame(){
  if (!$Game::Running)  {
    error("endGame: No game running!");
    return;
  }
   
  // Stop the AIManager
  //AIManager.delete();

  // Stop any game timers
  cancel($Game::Schedule);

  // Inform the client the game is over
  for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
    %cl = ClientGroup.getObject( %clientIndex );
    commandToClient(%cl, 'GameEnd');
  }

  // Delete all the temporary mission objects
  resetMission();
  $Game::Running = false;
}

//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------

function pickSpawnPoint(%checkpointNumber) {
  if (%checkpointNumber $= ""){
    %checkpointNumber = 0;
  }
   
  %groupName = "MissionGroup/Checkpoints";
  %group = nameToID(%groupName);

  if (%group != -1) {
    %count = %group.getCount();
    for (%i = 0; %i < %count; %i++){
      %thiscp = %group.getObject(%i);
      if (%thiscp.getClassName() $= "SpawnSphere"){
        if (%thiscp.checkpointNumber !$= ""){
     
          // is this the checkpoint we want
          if (%thiscp.checkpointNumber == %checkpointNumber){
            return %thiscp.getTransform();
          }
            
        // not a checkpoint
        }else{
          echo("Warning:  Invalid checkpoint in checkpoint group.");
        }
      // not a spawn sphere
      }else{
        echo("Warning:  Invalid checkpoint in checkpoint group.");
      }
    }
    
    // if we made it here, we could not find the correct checkpoint
    error("Could not find the correct checkpoint to spawn at in simgroup: " @ %groupName @ "  Wanted #" @ %checkpointNumber);
  
  // if missing checkpoints group
  }else{
    error("Missing checkpoints group " @ %groupName);
  }
  
  // Could be no spawn points, in which case we'll stick the
  // player at the center of the world.
  return "0 0 300 1 0 0 0";
}

//-----------------------------------------------------------------------------
