//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::onClientEnterGame(%this){
  //commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);

  // Create a new camera object.
  %this.camera = new Camera() {
    dataBlock = Observer;
  };
  MissionCleanup.add( %this.camera );
  %this.camera.scopeToClient(%this);

  // Setup game parameters, the onConnect method currently starts
  // everyone with a 0 score.
  %this.score = 0;

  // start at first checkpoint
  %this.lastCheckpoint = 0;
   
  // initialize the in-game user interface
  ingameui.initialize();
  %this.inventoryInitialize();
  
  // this creates the storyline handler object
  %this.storylineHandler = new ScriptObject(){
    class = storylineHandler;
  };
  
  // initialize the storyline handler
  %this.storylineHandler.initialize(%this);

  // Create a player object.
  %this.spawnPlayer();
  
  %this.storylineHandler.signal("reachCheckpoint",0);
  %this.savegamestate();
  
  //echo(%this.inventoryAdd("Testing"));
  //echo(%this.inventoryAdd("testing"));
  //echo(%this.inventoryContains("TestIng"));
  //%this.inventorySave();
  //echo(%this.inventoryRemove("Testing"));
  //echo(%this.inventoryRemove("aa"));
  //%this.inventoryRestore();
  //%this.dump();
}

//-----------------------------------------------------------------------------

function GameConnection::onClientLeaveGame(%this){
  if (isObject(%this.camera))
    %this.camera.delete();
  if (isObject(%this.player))
  %this.player.delete();
}

//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this){
  // The control objects invoked this method when they
  // move out of the mission area.
}

function GameConnection::onEnterMissionArea(%this){
  // The control objects invoked this method when they
  // move back into the mission area.
}


//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc){
  // Clear out the name on the corpse
  %this.player.setShapeName("");

  // Switch the client over to the death cam and unhook the player object.
  if (isObject(%this.camera) && isObject(%this.player)) {
    %this.setFirstPerson(false);
    %this.camera.setMode("Corpse",%this.player);
    %this.setControlObject(NULL);
  }
  %this.player = 0;
   
  if (%damageType $= "Suicide"){
    %this.schedule(7000, "spawnPlayer", true);
  }else{
    bottomPrintAll("\c4Mission Failed - You died.", 0);
    %this.schedule(10000, "spawnPlayer", true);
  }
}

//-----------------------------------------------------------------------------

function GameConnection::spawnPlayer(%this, %previouslyLived){
  // Combination create player and drop him somewhere
  %spawnPoint = pickSpawnPoint(%this.lastCheckPoint);
  %this.createPlayer(%spawnPoint);
  
  if (%previouslyLived == true){
    bottomPrintAll("\c4Restarting from last checkpoint....", 6);
    %this.restoreGameState();
  }
}   

//-----------------------------------------------------------------------------

function GameConnection::createPlayer(%this, %spawnPoint){
  if (%this.player > 0)  {
    // The client should not have a player currently
    // assigned.  Assigning a new one could result in 
    // a player ghost.
    error( "Attempting to create an angus ghost!" );
  }

  // Create the player object
  %player = new Player() {
    dataBlock = MainPlayerData;
    client = %this;
  };
  MissionCleanup.add(%player);

  // Player setup...
  %player.setTransform(%spawnPoint);
  %player.setShapeName(%this.name);
   
  // Update the camera to start with the player
  %this.camera.setMode("WithPlayer", %player);

  // Give the client control of the player
  %this.player = %player;
  %this.setControlObject(%player);

  // current weapon is none (which is invalid)
  %player.currentWeapon = -1;
  
  // switch to weapon 0
  %this.switchWeapon(0);
   
  $MainPlayer = %player;
}

//-----------------------------------------------------------------------------

// saves the game's state (for use when player reaches a checkpoint)
function GameConnection::saveGameState(%this){
  ingameUI.saveObjectives();
  %this.inventorySave();
  %this.storylineHandler.signal("save");
}

//-----------------------------------------------------------------------------

// restores the game's last state that was saved (for when player dies, we can put him back at the last checkpoint)
function GameConnection::restoreGameState(%this){
  ingameUI.restoreObjectives();
  %this.inventoryRestore();
  %this.storylineHandler.signal("restore");
}

//-----------------------------------------------------------------------------
// switches the current weapon
// 
// this function calls the Player::switchWeapon method
function GameConnection::switchWeapon(%this, %wNum){
  if ((%wNum > 1) || (%wNum < 0)){
    echo("GameConnection::switchWeapon() called with invalid wNum parameter!");
    return;    
  }
  
  %old = %this.player.currentWeapon;
  %new = %old;
  
  if (%old != %wNum){
    switch(%wNum){
      case 0:
        %this.player.switchWeapon(%wNum);
        %new = 0;
        //break;
      
      case 1:
        if (%this.inventoryContains("Neutralizer") == true){
          %this.player.switchWeapon(%wNum);
          %new = 1;  
        }else{
          bottomPrintAll("\c4You don't have the neutralizer yet.", 4);
        }
        //break;
      
      //case 2:
      //  %this.player.switchWeapon(%wNum);
      //  %new = 2;
      //  //break;
    }

    // if we really switched  
    if (%old != %new){
      commandToClient( %this, 'SwitchWeapon', %new );
    }
  // endif we really are trying to switch to a different choice
  }  
}

//----------------------------------------------------------------------------