//-----------------------------------------------------------------------------
//
// This object hierarchy exists for each rocket fired:
//
// RocketProjectile -> RocketExhaustParticleEmitter -> RocketExhaustParticle
//                  -> RocketExplosion (created upon impact, see below)
//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// If a rocket hits something, this object hierarchy will become active as
// each rocket will create an explosion effect upon impact:
//
// RocketExplosion -> RocketExplosionFireParticleEmitter   -> RocketExplosionFireParticle
//				   -> RocketExplosionSmokeParticleEmitter  -> RocketExplosionSmokeParticle
//				   -> RocketExplosionSparksParticleEmitter -> RocketExplosionSparksParticles
//
//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
// When the mind hits something, this is what it will sound like...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock AudioProfile( MindDropSound ){
//	filename = "~/data/sound/splish.wav";
	description = AudioDefault3d;
	preload = true;
};

  datablock AudioProfile( MindSpraySound ){
	filename = "~/data/sound/MindPush.wav";
	description = AudioCloseLooping3d;
	preload = true;
};

//-----------------------------------------------------------------------------

datablock ParticleData( MindProjectileHitParticle ){
	dragCoefficient    = 0.0;
  windCoefficient    = 0.0;
  gravityCoefficient = 0.2; //-0.015; // Gravity pulls down, but a negative value makes the particles rise instead of sink
	inheritedVelFactor = 1.00;
	constantAcceleration = 0.0;
	lifetimeMS = 500;
	lifetimeVarianceMS = 200;
  //spinSpeed = 
	spinRandomMin      = -30.0;
	spinRandomMax      = 30.0;
	useInvAlpha = false;
  animateTexture    = false;  
  textureName = "~/data/shapes/particletextures/spark.png";
  
	times[0] = 0.0;
	times[1] = 0.2;
	times[2] = 0.2;
  times[3] = 0.2;
  times[4] = 0.2;

	colors[0] = "0.8 0.2 0.2 0.1";
  colors[1] = "0.8 0.2 0.2 0.1";
  colors[2] = "0.8 0.2 0.2 0.1";
  colors[3] = "0.8 0.2 0.2 0.1";
  colors[4] = "0.8 0.2 0.2 0.1";

	sizes[0] = 0.01;
  sizes[1] = 0.05;
  sizes[2] = 0.05;
  sizes[3] = 0.05;
  sizes[4] = 0.01;
};

//-----------------------------------------------------------------------------

datablock ParticleEmitterData( MindProjectileHitParticleEmitter ){
	particles = "MindProjectileHitParticle";

	lifetimeMS = 1.0;
	lifetimeVarianceMS = 0;

	ejectionPeriodMS = 10;
	periodVarianceMS = 0;

	ejectionVelocity = 0.2;
	velocityVariance = 0.1;
  
  ejectionOffset = 0;
  thetaMin = 0.0;
  thetaMax = 0;
  phiReferenceVel = 0;
  phiVariance = 0;
  
};

//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// When a rocket hits something, it will explode like this...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ExplosionData( MindProjectileHitData ){
	soundProfile = MindDropSound;

	lifeTimeMS = 5000;
        lifetimeVarianceMS = 0;

};

//-----------------------------------------------------------------------------





// *******************************************************************************************
// ********End of Explosion-Related Datablocks, start of weapon/projectile datablocks***************
// *******************************************************************************************





//-----------------------------------------------------------------------------
// When a rocket is flying through the air, it will emit exhaust like this...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ParticleData( MindProjectileExhaustParticle ){
	dragCoefficient    = 0.0;
  windCoefficient    = 0.0;
  gravityCoefficient = 0.2; //-0.015; // Gravity pulls down, but a negative value makes the particles rise instead of sink
	inheritedVelFactor = 1.00;
	constantAcceleration = 0.0;
	lifetimeMS = 2000;
	lifetimeVarianceMS = 500;
  //spinSpeed = 
	spinRandomMin      = -30.0;
	spinRandomMax      = 30.0;
	useInvAlpha = false;
  animateTexture    = false;  
  textureName = "~/data/shapes/particletextures/spark.png";
	colors[0] = "1.0 0.0 0.2 0.0";
	colors[1] = "1.0 0.0 0.1 1.0";
	colors[2] = "1.0 0.0 0.1 0.7";
	colors[3] = "0.1 0.1 0.1 0.7";
	colors[4] = "0.1 0.1 0.1 0.0";
	
	sizes[0] = 0.05;
	sizes[1] = 0.05;
	sizes[2] = 0.05;
	sizes[3] = 0.025;
	sizes[4] = 0.25;
	times[0] = 0.0;
	times[1] = 0.2;
	times[2] = 0.7;
	times[3] = 1.2;
	times[4] = 1.7;
};

//-----------------------------------------------------------------------------

datablock ParticleEmitterData( MindProjectileExhaustParticleEmitter ){
	particles = MindProjectileExhaustParticle;

	ejectionPeriodMS = 1.0;
	periodVarianceMS = 0;

	ejectionVelocity = 3.0;
	velocityVariance = 2.0;
  
  lifetimeMS = 1500;
};

//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
// Projectile emitted when fired
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ProjectileData( MindProjectile ){
	projectileShapeName = "~/data/shapes/bullet/bullet.dts";
  scale="0.1 0.1 0.1";

	armingDelay    = 0;
   lifetime            = 5000;
   fadeDelay           = 5000;
//   bounceElasticity    = 0;
//   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod = 0.80;

   hasWaterLight     = false;

	directDamage = 20;
	radiusDamage = 10;
	damageRadius = 1.5;

	explosion    = MindProjectileHitData; 
	particleEmitter = MindProjectileExhaustParticleEmitter;

	muzzleVelocity = 40;
//   muzzleVelocity      = 1000;
   numProjectiles      = 1;
   spreadRatio		   = 1/150;
};

//-----------------------------------------------------------------------------

function MindProjectile::onCollision( %this, %obj, %col, %fade, %pos, %normal ){
//  echo("MindProjectile::onCollision called! ----------- " @ %col.getClassName() @ " " @ %col.getName());

   // Apply damage to the object all shape base objects
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
     if(!((%col.getClassName() $= "Player") || (%col.getName() $= "AllyPlayer"))){
       %col.damage(%obj,%pos,%this.directDamage,"MindProjectile");
    
//   radiusDamage(%obj, VectorAdd(%pos, VectorScale(%normal, 0.01)),
//        %this.damageRadius, 0, "Radius", %this.areaImpulse);
     }
}

//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
// Mind Weapon Datablock
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ShapeBaseImageData( MindWeapon ){
  shapeFile = "~/data/shapes/bullet/bullet.dts";
  //scale="0.001 0.001 0.001";
  
  projectile = MindProjectile;
  projectileType = Projectile;
  fireTimeout = 0;
  emap = true;

  // When firing from a point offset from the eye, muzzle correction will 
  // adjust the muzzle vector to point to the eye LOS point. Since this 
  // weapon doesn't actually fire from the muzzle point, we need to turn 
  // this off.
  correctMuzzleVector = false;

  // Specify mountPoint & offset for 3rd person, and eyeOffset for first 
  // person rendering.
  mountPoint = 0;
  offset = "0.0 0.0 0.0";
  eyeOffset = ".143 .102 -.212";
   
  // Images have a state system which controls how the animations are run, 
  // which sounds are played, script callbacks, etc. This state system is 
  // downloaded to the client so that clients can predict state changes and 
  // animate accordingly.  The following system supports basic 
  // ready->fire->reload transitions as well as a no-ammo->dryfire idle 
  // state.

  // Initial start up state
  stateName[0]                     = "Preactivate";
  stateTransitionOnLoaded[0]       = "Activate";
  stateTransitionOnNoAmmo[0]       = "NoAmmo";

  // Activating the gun. Called when the weapon is first mounted and there is ammo.
  stateName[1]                     = "Activate";
  stateTransitionOnTimeout[1]      = "Ready";
  stateTimeoutValue[1]             = 0.5;
  stateSequence[1]                 = "Activate";

  // Ready to fire, just waiting for the trigger
  stateName[2]                     = "Ready";
  stateTransitionOnNoAmmo[2]       = "NoAmmo";
  stateTransitionOnTriggerDown[2]  = "Fire";

  // Fire the weapon. Calls the onFire function when our player shoots.
  stateName[3]                     = "Fire";
  stateTransitionOnTimeout[3]      = "Ready";
  stateTimeoutValue[3]             = 0.07;
  stateFire[3]                     = true;
  stateRecoil[3]                   = LightRecoil;
  stateAllowImageChange[3]         = false;
  stateSequence[3]                 = "Fire";
  stateScript[3]                   = "onFire";
  stateEmitterTime[3]              = 0.3;

  // Play the relead animation, and transition into
  //stateName[4]                     = "Reload";
  //stateTransitionOnNoAmmo[4]       = "NoAmmo";
  //stateTransitionOnTimeout[4]      = "Ready";
  //stateTimeoutValue[4]             = 0.1;
  //stateAllowImageChange[4]         = false;
  //stateSequence[4]                 = "Reload";
  //stateEjectShell[4]               = true;

  // No ammo in the weapon, just idle until something shows up. Play the dry 
  // fire sound if the trigger is pulled.
  stateName[5]                     = "NoAmmo";
  stateTransitionOnAmmo[5]         = "Ready";
  stateSequence[5]                 = "NoAmmo";
  stateTransitionOnTriggerDown[5]  = "DryFire";

  // No ammo! Just perform a dry fire until the player finds more ammo.
  stateName[6]                     = "DryFire";
  stateTimeoutValue[6]             = 1.0;
  stateTransitionOnTimeout[6]      = "NoAmmo";
};

//-----------------------------------------------------------------------------

function MindWeapon::onFire( %this, %obj, %slot ){
  %projectile = %this.projectile;
  if (%obj.mindpushLevel < 0){
    bottomPrintAll("\c4Mindpush is recharging...", 2);
    return;
  }

   // Determine initial projectile velocity based on the 
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));
       
   for (%a = 0;%a<%projectile.numProjectiles;%a++)
   {  
		%zcomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("0 0 1", (-100 + getRandom() *200));
		%ycomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("0 1 0", (-100 + getRandom() *200));
		%xcomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("1 0 0", (-100 + getRandom() *200));
		   
	    %spreadVector = MatrixCreateFromEuler(%xcomp @ " " @ %ycomp @ " " @ %zcomp);
		   
		%Velocity = MatrixMulVector(%spreadVector,%muzzleVelocity);

		// Create the projectile object

		%p = new (%this.projectileType)() {
			dataBlock        = %projectile;
			initialVelocity  = %Velocity;
			initialPosition  = %obj.getMuzzlePoint(%slot);
			sourceObject     = %obj;
			sourceSlot       = %slot;
			client           = %obj.client;
		};
   }

    // does spraying audio
    // this is how it works, essentially, we start playing the looping sound but we schedule it to be stopped in 150 ms.
    // however, if we reach this point while there is a stop call scheduled, we cancel the schedule and then reschedule the stopping of the sound effect
    // that way, while you have the button pressed, the sound is constantly looping until you release the button
     if (%this.schedStop !$= ""){
       cancel(%this.schedStop);
       %this.schedStop = "";
     }else{
       %obj.playAudio(2, MindSpraySound);
     }
     %this.schedStop = %this.schedule(150, stopSound, %obj);

  // this section does the limited amount of spraying code
 
   // reduce mindpush level 
   %obj.mindpushLevel -= 10; 
   // schedule refill of sprayer if not already scheduled
   if (%this.schedRefill $= "")
     %this.schedRefill = %this.schedule(10000, refill, %obj);

   MissionCleanup.add(%p);
   
   return %p;
}

//-----------------------------------------------------------------------------
function MindWeapon::stopSound(%this, %obj){
  %this.schedStop = "";
  %obj.stopAudio(2);
}

//-----------------------------------------------------------------------------
// refills the brain power a bit
function MindWeapon::refill(%this, %obj){
  %this.schedRefill = "";
  %obj.mindpushLevel = 100;
}

//-----------------------------------------------------------------------------
