//-----------------------------------------------------------------------------
//
// This object hierarchy exists for each rocket fired:
//
// RocketProjectile -> RocketExhaustParticleEmitter -> RocketExhaustParticle
//                  -> RocketExplosion (created upon impact, see below)
//
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
//
// If a rocket hits something, this object hierarchy will become active as
// each rocket will create an explosion effect upon impact:
//
// RocketExplosion -> RocketExplosionFireParticleEmitter   -> RocketExplosionFireParticle
//				   -> RocketExplosionSmokeParticleEmitter  -> RocketExplosionSmokeParticle
//				   -> RocketExplosionSparksParticleEmitter -> RocketExplosionSparksParticles
//
//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
// When the neutralizer hits something, this is what it will sound like...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock AudioProfile( DropletSound ){
	filename = "~/data/sound/splish.wav";
	description = AudioDefault3d;
	preload = true;
};

  datablock AudioProfile( SpraySound ){
	filename = "~/data/sound/waterspray.wav";
	description = AudioCloseLooping3d;
	preload = true;
};

//-----------------------------------------------------------------------------

datablock ParticleData( WaterProjectileHitParticle ){
	dragCoefficient    = 0.0;
  windCoefficient    = 0.0;
  gravityCoefficient = 0.5; //-0.015; // Gravity pulls down, but a negative value makes the particles rise instead of sink
	inheritedVelFactor = 1.00;
	constantAcceleration = 0.0;
	lifetimeMS = 500;
	lifetimeVarianceMS = 200;
  //spinSpeed = 
	spinRandomMin      = -30.0;
	spinRandomMax      = 30.0;
	useInvAlpha = false;
  animateTexture    = false;  
  textureName = "~/data/shapes/particletextures/spark.png";
  
	times[0] = 0.0;
	times[1] = 0.2;
	times[2] = 0.4;
  times[3] = 0.6;
  times[4] = 0.8;

	colors[0] = "0.8 0.8 1.0 0.1";
  colors[1] = "0.8 0.8 1.0 0.1";
  colors[2] = "0.8 0.8 1.0 0.1";
  colors[3] = "0.8 0.8 1.0 0.1";
  colors[4] = "0.8 0.8 1.0 0.1";

	sizes[0] = 0.01;
  sizes[1] = 0.05;
  sizes[2] = 0.08;
  sizes[3] = 0.05;
  sizes[4] = 0.01;
};

//-----------------------------------------------------------------------------

datablock ParticleEmitterData( WaterProjectileHitParticleEmitter ){
	particles = "WaterProjectileHitParticle";

	lifetimeMS = 1.0;
	lifetimeVarianceMS = 0;

	ejectionPeriodMS = 10;
	periodVarianceMS = 0;

	ejectionVelocity = 0.2;
	velocityVariance = 0.1;
  
  ejectionOffset = 0;
  thetaMin = 0.0;
  thetaMax = 0;
  phiReferenceVel = 0;
  phiVariance = 0;
  
};

//-----------------------------------------------------------------------------



//-----------------------------------------------------------------------------
// When a rocket hits something, it will explode like this...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ExplosionData( WaterProjectileHitData ){
	soundProfile = DropletSound;

	lifeTimeMS = 5000;
  lifetimeVarianceMS = 0;

	// Volume particles
	//particleEmitter = WaterProjectileHitParticleEmitter;
	//particleDensity = 15;
	//particleRadius  = 2.5;

	// Point emission particles
	//`emitter[0] = WaterProjectileHitParticleEmitter;
	//emitter[1] = RocketExplosionSparksParticleEmitter;

	// This will make the camera shake when a player gets hit by a rocket.
  // Shoot your own feet to see this effect in action.
	shakeCamera      = false;
	//camShakeFreq     = "10.0 11.0 10.0";
	//camShakeAmp      = "1.0 1.0 1.0";
	//camShakeDuration = 0.5;
	//camShakeRadius   = 10.0;

	// This will create a dynamic lighting effect in the vicinity of the rocket's explosion.
	//lightStartRadius = 5;
	//lightEndRadius   = 2;
	//lightStartColor  = "0.0 0.0 0.8";
	//lightEndColor    = "0.0 0.0 0.0";
};

//-----------------------------------------------------------------------------





// *******************************************************************************************
// ********End of Explosion-Related Datablocks, start of weapon/projectile datablocks***************
// *******************************************************************************************





//-----------------------------------------------------------------------------
// When a rocket is flying through the air, it will emit exhaust like this...
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ParticleData( WaterProjectileExhaustParticle ){
	dragCoefficient    = 0.2;
  windCoefficient    = 0.0;
  gravityCoefficient = 0.5; //-0.015; // Gravity pulls down, but a negative value makes the particles rise instead of sink
	inheritedVelFactor = 1.00;
	constantAcceleration = 0.0;
	lifetimeMS = 1000;
	lifetimeVarianceMS = 500;
  //spinSpeed = 
	spinRandomMin      = -30.0;
	spinRandomMax      = 30.0;
	useInvAlpha = false;
  animateTexture    = false;  
  textureName = "~/data/shapes/particletextures/spark.png";
	colors[0] = "0.2 0.0 1.0 0.0";
	colors[1] = "0.1 0.0 1.0 1.0";
	colors[2] = "0.1 0.0 1.0 0.7";
	colors[3] = "0.1 0.1 0.1 0.7";
	colors[4] = "0.1 0.1 0.1 0.0";
	
	sizes[0] = 0.05;
	sizes[1] = 0.05;
	sizes[2] = 0.05;
	sizes[3] = 0.025;
	sizes[4] = 0.25;
	times[0] = 0.0;
	times[1] = 0.2;
	times[2] = 0.7;
	times[3] = 1.2;
	times[4] = 1.7;
};

//-----------------------------------------------------------------------------

datablock ParticleEmitterData( WaterProjectileExhaustParticleEmitter ){
	particles = WaterProjectileExhaustParticle;

	ejectionPeriodMS = 1.0;
	periodVarianceMS = 0;

	ejectionVelocity = 3.0;
	velocityVariance = 2.0;
  
  lifetimeMS = 1500;
};

//-----------------------------------------------------------------------------





//-----------------------------------------------------------------------------
// Projectile emitted when fired
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ProjectileData( WaterProjectile ){
	projectileShapeName = "~/data/shapes/bullet/bullet.dts";
  scale="0.1 0.1 0.1";

	muzzleVelocity = 8;
	armingDelay    = 0;
	lifetime       = 5000;
	fadeDelay      = 5000;
	isBallistic    = true;
	gravityMod     = 0.8;

	hasLight    = false;
	//lightRadius = 5;
	//lightColor  = "0.5 0.5 0.25";

  // The rocket will emit exhaust particles using this
	particleEmitter = WaterProjectileExhaustParticleEmitter;

  // We'll create one of these if our rocket hits something
	explosion    = WaterProjectileHitData; 
	directDamage = 20;
	radiusDamage = 10;
	damageRadius = 1.5;
};

//-----------------------------------------------------------------------------

function WaterProjectile::onCollision( %this, %obj, %col, %fade, %pos, %normal ){
 
  // check for puddle collisions
  // we have to do this because Torque is rather stupid in that onCollision() is not called 
  // for shapeBase objects unless it is collided with an object that will do the call for it
  // 
  // for instance, Projectile--Item cannot result in an onCollision call, but a Player--Item collision does 
  // result in a onCollision call because the Player object calls the onCollision method of the Item.
  // very strange design
  //
  // so, in short, we have to manually check for any puddle collsions here
  // we can do this because we know that puddles are on the "ground"
  // since onCollision for the water "bullet" IS called when we hit the ground, just not when we hit the puddle
  // however, since the puddle is on the ground, we know that the bullet on the ground will be right near the puddle
  // if we really did "hit" it. 
  
  // we assume all puddles are in one group
  %groupName = PuddleData.puddleGroupName;
  %group = nameToID(%groupName);

  if (%group != -1) {
    %count = %group.getCount();
    // for each object in group
    for (%i = 0; %i < %count; %i++){
      %thisob = %group.getObject(%i);
      if (%thisob.getClassName() $= "Item"){
        %db = %thisob.getDatablock();
        if (%db.getName() $= "PuddleData"){
        
          %pt = %obj.getWorldBoxCenter();

          // if we are colliding          
          if (%db.isPointColliding(%thisob, %pt) == true){
            %db.bulletHit(%thisob);
            return;
          }
     

        } // not a puddle
      } // not an Item
    } // for each item in puddle group
      
  // if missing puddles group
  }else{
    error("Missing puddle group " @ %groupName);
  }
  // end checking for puddle collisions

  // Check for AI Hits
  if (%col.getType() & $TypeMasks::ShapeBaseObjectType) {
     if(!((%col.getClassName() $= "Player") || (%col.getName() $= "AllyPlayer"))) {
       %col.damage(%obj,%pos,%this.directDamage,"WaterProjectile");
     }
  }
}

//-----------------------------------------------------------------------------




//-----------------------------------------------------------------------------
// Neutralizer Weapon Datablock
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

datablock ShapeBaseImageData( NeutralizerWeapon ){
  shapeFile = "~/data/shapes/pistol/handgun.dts";
  
  projectile = WaterProjectile;
  projectileType = Projectile;
  fireTimeout = 0;
  emap = true;

  // When firing from a point offset from the eye, muzzle correction will 
  // adjust the muzzle vector to point to the eye LOS point. Since this 
  // weapon doesn't actually fire from the muzzle point, we need to turn 
  // this off.
  correctMuzzleVector = false;

  // Specify mountPoint & offset for 3rd person, and eyeOffset for first 
  // person rendering.
  mountPoint = 0;
  offset = "0.0 0.0 0.0";
  eyeOffset = "0.32 0.0 -0.41";

  // Images have a state system which controls how the animations are run, 
  // which sounds are played, script callbacks, etc. This state system is 
  // downloaded to the client so that clients can predict state changes and 
  // animate accordingly.  The following system supports basic 
  // ready->fire->reload transitions as well as a no-ammo->dryfire idle 
  // state.

  // Initial start up state
  stateName[0]                     = "Preactivate";
  stateTransitionOnLoaded[0]       = "Activate";
  stateTransitionOnNoAmmo[0]       = "NoAmmo";

  // Activating the gun. Called when the weapon is first mounted and there is ammo.
  stateName[1]                     = "Activate";
  stateTransitionOnTimeout[1]      = "Ready";
  stateTimeoutValue[1]             = 0.5;
  stateSequence[1]                 = "Activate";

  // Ready to fire, just waiting for the trigger
  stateName[2]                     = "Ready";
  stateTransitionOnNoAmmo[2]       = "NoAmmo";
  stateTransitionOnTriggerDown[2]  = "Fire";

  // Fire the weapon. Calls the onFire function when our player shoots.
  stateName[3]                     = "Fire";
  stateTransitionOnTimeout[3]      = "Ready";
  stateTimeoutValue[3]             = 0.07;
  stateFire[3]                     = true;
  stateRecoil[3]                   = LightRecoil;
  stateAllowImageChange[3]         = false;
  stateSequence[3]                 = "Fire";
  stateScript[3]                   = "onFire";
  stateEmitterTime[3]              = 0.3;

  // Play the relead animation, and transition into
  //stateName[4]                     = "Reload";
  //stateTransitionOnNoAmmo[4]       = "NoAmmo";
  //stateTransitionOnTimeout[4]      = "Ready";
  //stateTimeoutValue[4]             = 0.1;
  //stateAllowImageChange[4]         = false;
  //stateSequence[4]                 = "Reload";
  //stateEjectShell[4]               = true;

  // No ammo in the weapon, just idle until something shows up. Play the dry 
  // fire sound if the trigger is pulled.
  stateName[5]                     = "NoAmmo";
  stateTransitionOnAmmo[5]         = "Ready";
  stateSequence[5]                 = "NoAmmo";
  stateTransitionOnTriggerDown[5]  = "DryFire";

  // No ammo! Just perform a dry fire until the player finds more ammo.
  stateName[6]                     = "DryFire";
  stateTimeoutValue[6]             = 1.0;
  stateTransitionOnTimeout[6]      = "NoAmmo";
};

//-----------------------------------------------------------------------------

function NeutralizerWeapon::onFire( %this, %obj, %slot ){
  // if our sprayer level is too low, we don't fire
  if (%obj.sprayerLevel < 0){
    bottomPrintAll("\c4Refilling your sprayer...", 2);
    return;
  }
  
  %projectile = %this.projectile;

	// Determine initial projectile velocity based on the gun's muzzle point 
	// and the object's current velocity...
	%muzzleVector   = %obj.getMuzzleVector( %slot );
	%objectVelocity = %obj.getVelocity();
	
  %muzzleVector = VectorAdd(%muzzleVector, "0 0 0.2"); 

	%muzzleVelocity = VectorAdd( VectorScale(%muzzleVector, %projectile.muzzleVelocity),
                               VectorScale(%objectVelocity, %projectile.velInheritFactor) );

	// Create a new rocket projectile object...
	%p = new (%this.projectileType)(){
		dataBlock       = %projectile;
		initialVelocity = %muzzleVelocity;
		initialPosition = %obj.getMuzzlePoint(%slot);
		sourceObject    = %obj;
		sourceSlot      = %slot;
		client          = %obj.client;
	};
  
  
  // does spraying audio
  // this is how it works, essentially, we start playing the looping sound but we schedule it to be stopped in 150 ms.
  // however, if we reach this point while there is a stop call scheduled, we cancel the schedule and then reschedule the stopping of the sound effect
  // that way, while you have the button pressed, the sound is constantly looping until you release the button
  if (%this.schedStop !$= ""){
    cancel(%this.schedStop);
    %this.schedStop = "";
  }else{
    %obj.playAudio(2, SpraySound);
  }
  %this.schedStop = %this.schedule(150, stopSound, %obj);
  
  
  // this section does the limited amount of spraying code
 
  // reduce sprayer level 
  %obj.sprayerLevel -= 2; 
  // schedule refill of sprayer if not already scheduled
  if (%obj.schedRefill $= ""){
    %obj.schedRefill = %obj.schedule(2000, refillSprayer, %obj);
  }
  
	MissionCleanup.add( %p );
	return %p;
}

//-----------------------------------------------------------------------------
// used by the fire() method to schedule that the spraying sound be stopped
function NeutralizerWeapon::stopSound(%this, %obj){
  %this.schedStop = "";
  %obj.stopAudio(2);
}

//-----------------------------------------------------------------------------
