//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------
// Pistol weapon. This file contains all the items related to this weapon
// including explosions, ammo, the item and the weapon item image.
// These objects rely on the item & inventory support system defined
// in item.cs and inventory.cs
//-----------------------------------------------------------------------------
// Based on Noon of the Dead pistol
//-----------------------------------------------------------------------------
// Sounds profiles

datablock AudioProfile(PistolReloadSound)
{
filename = "";  // not using reload sound
description = AudioClose3d;
preload = false;
};

datablock AudioProfile(PistolFireSound)
{
filename = "~/data/sound/handgunfire2.ogg";
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(PistolFireEmptySound)
{
filename = "";  // not using fire empty sound
description = AudioClose3d;
preload = true;
};

datablock AudioProfile(PistolExplosionSound)
{
filename = "";   // not using explosion sound
description = AudioDefault3d;
preload = true;
};


datablock ExplosionData(PistolExplosion)
{
   soundProfile = PistolExplosionSound;

   textureName          = "~/data/shapes/pistol/smoke";
   dragCoeffiecient     = 3.0;
   gravityCoefficient   = -0.1;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.00;
   lifetimeMS           = 300;
   lifetimeVarianceMS   = 150;

   colors[0]     = "0.3 0.3 0.3 0.7";
   colors[1]     = "0.2 0.2 0.2 0.4";
   colors[2]     = "0.1 0.1 0.1 0.0";

   sizes[0]      = 1.0;
   sizes[1]      = 0.66;
   sizes[2]      = 0.33;

   times[0]      = 0.0;
   times[1]      = 0.5;
   times[2]      = 1.0;
};

//-----------------------------------------------------------------------------
// Projectile Object

datablock ProjectileData(PistolProjectile)
{
   projectileShapeName = "~/data/shapes/Bullet/Bullet.dts";

   lifetime            = 1000;
   fadeDelay           = 1000;
   bounceElasticity    = 0;
   bounceFriction      = 0;
   isBallistic         = true;
   gravityMod = 0.80;

   hasWaterLight     = false;

	directDamage = 20;
	radiusDamage = 10;
	damageRadius = 1.5;

   explosion           = PistolExplosion;

   muzzleVelocity      = 1000;
   numProjectiles      = 1;
   spreadRatio		   = 1/150;
};

function PistolProjectile::onCollision(%this,%obj,%col,%fade,%pos,%normal)
{
   // Apply damage to the object all shape base objects
   if (%col.getType() & $TypeMasks::ShapeBaseObjectType)
       %col.damage(%obj,%pos,%this.directDamage,"Bullet");
    
//   radiusDamage(%obj, VectorAdd(%pos, VectorScale(%normal, 0.01)),
//        %this.damageRadius, 0, "Radius", %this.areaImpulse);
}


//--------------------------------------------------------------------------
// Weapon Item.  This is the item that exists in the world, i.e. when it's
// been dropped, thrown or is acting as re-spawnable item.  When the weapon
// is mounted onto a shape, the PistolImage is used.
/*
datablock ItemData(Pistol)
{
   // Mission editor category
   category = "Weapon";

   // Hook into Item Weapon class hierarchy. The weapon namespace
   // provides common weapon handling functions in addition to hooks
   // into the inventory system.
   className = "Weapon";

   // Basic Item properties
   shapeFile = "~/data/shapes/pistol/Tommygun.dts";
   mass = 1;
   elasticity = 0.2;
   friction = 0.6;
   emap = true;
};
*/
//--------------------------------------------------------------------------
// Pistol image which does all the work.  Images do not normally exist in
// the world, they can only be mounted on ShapeBase objects.

datablock ShapeBaseImageData(PistolImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/pistol/Tommygun.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0.0 0.0 0.0";
   eyeOffset = ".143 .102 -.212";

   // When firing from a point offset from the eye, muzzle correction
   // will adjust the muzzle vector to point to the eye LOS point.
   // Since this weapon doesn't actually fire from the muzzle point,
   // we need to turn this off.  
   correctMuzzleVector = false;

   // Add the WeaponImage namespace as a parent, WeaponImage namespace
   // provides some hooks into the inventory system.
   className = "WeaponImage";

   // Projectile && Ammo.
   item = Pistol;
   ammo = PistolAmmo;
   projectile = PistolProjectile;
   projectileType = Projectile;

   // Images have a state system which controls how the animations
   // are run, which sounds are played, script callbacks, etc. This
   // state system is downloaded to the client so that clients can
   // predict state changes and animate accordingly.  The following
   // system supports basic ready->fire->reload transitions as
   // well as a no-ammo->dryfire idle state.

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "NoAmmo";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.8;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "NoAmmo";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.6;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Shoot";
   stateScript[3]                   = "onFire";
   stateSound[3]                    = PistolFireSound;

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "NoAmmo";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.03;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;
   stateSound[4]                    = PistolReloadSound;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "NoAmmo";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "NoAmmo";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "NoAmmo";
   stateSound[6]                    = PistolFireEmptySound;
};


//-----------------------------------------------------------------------------

function PistolImage::onFire(%this, %obj, %slot)
{
   %projectile = %this.projectile;

   // Determine initial projectile velocity based on the 
   // gun's muzzle point and the object's current velocity
   %muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),
      VectorScale(%objectVelocity, %projectile.velInheritFactor));
       
   for (%a = 0;%a<%projectile.numProjectiles;%a++)
   {  
		%zcomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("0 0 1", (-100 + getRandom() *200));
		%ycomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("0 1 0", (-100 + getRandom() *200));
		%xcomp = (getRandom() - 0.5) * 2 * 3.1415926 * %projectile.spreadRatio; //VectorScale("1 0 0", (-100 + getRandom() *200));
		   
	    %spreadVector = MatrixCreateFromEuler(%xcomp @ " " @ %ycomp @ " " @ %zcomp);
		   
		%Velocity = MatrixMulVector(%spreadVector,%muzzleVelocity);

		// Create the projectile object

		%p = new (%this.projectileType)() {
			dataBlock        = %projectile;
			initialVelocity  = %Velocity;
			initialPosition  = %obj.getMuzzlePoint(%slot);
			sourceObject     = %obj;
			sourceSlot       = %slot;
			client           = %obj.client;
		};
		MissionCleanup.add(%p);
	}
   
   return %p;
}
