//-----------------------------------------------------------------------------
// Torque Game Engine 
// Player Stats File
// Much of this file was copied from the www.codesampler.com tutorial
//-----------------------------------------------------------------------------


// Load dts shapes and merge animations
exec("~/data/shapes/player/player.cs");


//----------------------------------------------------------------------------
// Player Sounds
//----------------------------------------------------------------------------

datablock AudioProfile(DeathCrySound){
  //fileName = "~/data/sound/orc_death.ogg";
  fileName = "~/data/sound/Insectoid_Death.ogg";
  description = AudioClose3d;
  preload = true;
};

datablock AudioProfile(PainCrySound){
  //fileName = "~/data/sound/orc_pain.ogg";
  fileName = "~/data/sound/Insectoid_Pain.ogg";
  description = AudioClose3d;
  preload = true;
};

datablock AudioProfile(FootLightSoftSound){
   filename    = "~/data/sound/stepsoft.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(FootLightHardSound){
   filename    = "~/data/sound/stephard.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightMetalSound){
   filename    = "~/data/sound/stepmetal.ogg";
   description = AudioClose3d;
   preload = true;
};

datablock AudioProfile(FootLightSnowSound){
   filename    = "~/data/sound/stepsoft.wav";
   description = AudioClosest3d;
   preload = true;
};

datablock AudioProfile(ImpactLightSoftSound){
   filename    = "~/data/sound/stepsoft.wav";
   description = AudioClose3d;
   preload = true;
   effect = ImpactSoftEffect;
};

datablock AudioProfile(ImpactLightHardSound){
   filename    = "~/data/sound/stephard.ogg";
   description = AudioClose3d;
   preload = true;
   effect = ImpactHardEffect;
};

datablock AudioProfile(ImpactLightMetalSound){
   filename    = "~/data/sound/stepmetal.ogg";
   description = AudioClose3d;
   preload = true;
   effect = ImpactMetalEffect;
};

datablock AudioProfile(ImpactLightSnowSound){
   filename    = "~/data/sound/stepsoft.wav";
   description = AudioClosest3d;
   preload = true;
   effect = ImpactSnowEffect;
};


//----------------------------------------------------------------------------
// Foot puffs
//----------------------------------------------------------------------------
datablock ParticleData(LightPuff){
   dragCoefficient      = 2.0;
   gravityCoefficient   = -0.01;
   inheritedVelFactor   = 0.6;
   constantAcceleration = 0.0;
   lifetimeMS           = 800;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = true;
   spinRandomMin        = -35.0;
   spinRandomMax        = 35.0;
   colors[0]     = "1.0 1.0 1.0 1.0";
   colors[1]     = "1.0 1.0 1.0 0.0";
   sizes[0]      = 0.1;
   sizes[1]      = 0.8;
   times[0]      = 0.3;
   times[1]      = 1.0;
};

datablock ParticleEmitterData(LightPuffEmitter){
   ejectionPeriodMS = 35;
   periodVarianceMS = 10;
   ejectionVelocity = 0.2;
   velocityVariance = 0.1;
   ejectionOffset   = 0.0;
   thetaMin         = 20;
   thetaMax         = 60;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   useEmitterColors = true;
   particles = "LightPuff";
};


//----------------------------------------------------------------------------
// Liftoff dust
//----------------------------------------------------------------------------
datablock ParticleData(LiftoffDust){
   dragCoefficient      = 1.0;
   gravityCoefficient   = -0.01;
   inheritedVelFactor   = 0.0;
   constantAcceleration = 0.0;
   lifetimeMS           = 1000;
   lifetimeVarianceMS   = 100;
   useInvAlpha          = true;
   spinRandomMin        = -90.0;
   spinRandomMax        = 500.0;
   colors[0]     = "1.0 1.0 1.0 1.0";
   sizes[0]      = 1.0;
   times[0]      = 1.0;
};

datablock ParticleEmitterData(LiftoffDustEmitter){
   ejectionPeriodMS = 5;
   periodVarianceMS = 0;
   ejectionVelocity = 2.0;
   velocityVariance = 0.0;
   ejectionOffset   = 0.0;
   thetaMin         = 90;
   thetaMax         = 90;
   phiReferenceVel  = 0;
   phiVariance      = 360;
   overrideAdvance = false;
   useEmitterColors = true;
   particles = "LiftoffDust";
};


//----------------------------------------------------------------------------
// Foot Prints
//----------------------------------------------------------------------------
datablock DecalData(LightMaleFootprint){
   sizeX       = 0.125;
   sizeY       = 0.25;
   textureName = "~/data/shapes/player/footprint";
};


datablock DebrisData( PlayerDebris ){
   explodeOnMaxBounce = false;

   elasticity = 0.15;
   friction = 0.5;

   lifetime = 4.0;
   lifetimeVariance = 0.0;

   minSpinSpeed = 40;
   maxSpinSpeed = 600;

   numBounces = 5;
   bounceVariance = 0;

   staticOnMaxBounce = true;
   gravModifier = 1.0;

   useRadiusMass = true;
   baseRadius = 1;

   velocity = 20.0;
   velocityVariance = 12.0;
};             



//----------------------------------------------------------------------------
// Actual Player Statistics
//----------------------------------------------------------------------------
datablock PlayerData(MainPlayerData){
   className = Armor;

   // we do not render our player mesh if we are in first person view
   renderFirstPerson = false;
   emap = true;
   
   // location of player mesh
   shapeFile = "~/data/shapes/player/player.dts";
   cameraMaxDist = 3;
   computeCRC = true;
  
   canObserve = true;
   cmdCategory = "Clients";

   cameraDefaultFov = 90.0;
   cameraMinFov = 5.0;
   cameraMaxFov = 120.0;
   
   debrisShapeName = "~/data/shapes/player/debris_player.dts";
   debris = playerDebris;

   aiAvoidThis = true;

   minLookAngle = -1.4;
   maxLookAngle = 1.4;
   maxFreelookAngle = 3.0;

   mass = 90;
   drag = 0.3;
   maxdrag = 0.4;
   density = 10;
   maxEnergy =  60;
   energyPerDamagePoint = 75.0;

   // damage statistics
   maxDamage = 100;
   //repairRate = 0.15;  // 4.8 per second
   
   rechargeRate = 0.256;

   runForce = 48 * 90;
   runEnergyDrain = 0;
   minRunEnergy = 0;
   maxForwardSpeed = 14;
   maxBackwardSpeed = 13;
   maxSideSpeed = 13;

   maxUnderwaterForwardSpeed = 8.4;
   maxUnderwaterBackwardSpeed = 7.8;
   maxUnderwaterSideSpeed = 7.8;

   jumpForce = 8.3 * 190;
   jumpEnergyDrain = 0;
   minJumpEnergy = 0;
   jumpDelay = 15;

   recoverDelay = 9;
   recoverRunForceScale = 1.2;

   minImpactSpeed = 45;
   speedDamageScale = 0.4;

   boundingBox = "1.2 1.2 2.3";
   pickupRadius = 0.75;
   
   // Damage location details
   boxNormalHeadPercentage       = 0.83;
   boxNormalTorsoPercentage      = 0.49;
   boxHeadLeftPercentage         = 0;
   boxHeadRightPercentage        = 1;
   boxHeadBackPercentage         = 0;
   boxHeadFrontPercentage        = 1;

   
   
   
   // Foot Prints
   decalData   = LightMaleFootprint;
   decalOffset = 0.25;
   
   // Foot puffs
   footPuffEmitter = LightPuffEmitter;
   footPuffNumParts = 10;
   footPuffRadius = 0.25;
   
   // dust emitter (what does it do?)
   dustEmitter = LiftoffDustEmitter;

   // Splash Properties (Commented out because we don't need them)
   //splash = PlayerSplash;
   //splashVelocity = 4.0;
   //splashAngle = 67.0;
   //splashFreqMod = 300.0;
   //splashVelEpsilon = 0.60;
   //bubbleEmitTime = 0.4;
   //splashEmitter[0] = PlayerFoamDropletsEmitter;
   //splashEmitter[1] = PlayerFoamEmitter;
   //splashEmitter[2] = PlayerBubbleEmitter;
   //mediumSplashSoundVelocity = 10.0;   
   //hardSplashSoundVelocity = 20.0;   
   //exitSplashSoundVelocity = 5.0;
   //footstepSplashHeight = 0.35;
   
   
   
   
   // Controls over slope of runnable/jumpable surfaces
   runSurfaceAngle  = 70;
   jumpSurfaceAngle = 80;

   minJumpSpeed = 20;
   maxJumpSpeed = 30;

   horizMaxSpeed = 68;
   horizResistSpeed = 33;
   horizResistFactor = 0.35;

   upMaxSpeed = 80;
   upResistSpeed = 25;
   upResistFactor = 0.3;
   
   
   
   
   // Footstep Sounds
   FootSoftSound        = FootLightSoftSound;
   FootHardSound        = FootLightHardSound;
   FootMetalSound       = FootLightMetalSound;
   FootSnowSound        = FootLightSnowSound;

   // impact sounds
   impactSoftSound      = ImpactLightSoftSound;
   impactHardSound      = ImpactLightHardSound;
   impactMetalSound     = ImpactLightMetalSound;
   impactSnowSound      = ImpactLightSnowSound;

   groundImpactMinSpeed    = 10.0;
   groundImpactShakeFreq   = "4.0 4.0 4.0";
   groundImpactShakeAmp    = "1.0 1.0 1.0";
   groundImpactShakeDuration = 0.8;
   groundImpactShakeFalloff = 10.0;
   
   observeParameters = "0.5 4.5 4.5";
};

//----------------------------------------------------------------------------





//----------------------------------------------------------------------------
// MainPlayer Datablock methods
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function MainPlayerData::onAdd(%this,%obj){
  %obj.ourConstantRepairRate = 0.025;
  //echo("MainPlayerData::onAdd()");
  %obj.setRepairRate(%obj.ourConstantRepairRate);

  %obj.sprayerLevel = 100; // 100% full to start with
  
  
  %obj.mindpushLevel = 100; // 100% full to start with
  // MAH - Added below from 20071209 merge, caused problems without
  %obj.currentWeapon = 0;
}

//----------------------------------------------------------------------------

function MainPlayerData::onRemove(%this, %obj){
  //echo("MainPlayerData::onRemove()");
  if (%obj.client.player == %obj){
    %obj.client.player = 0;
  }
}

//----------------------------------------------------------------------------

function MainPlayerData::onNewDataBlock(%this,%obj){
  // do nothing
}

//----------------------------------------------------------------------------





//----------------------------------------------------------------------------
// Player object methods
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function Player::playDeathAnimation( %this ){
  %this.setActionThread( "die" );
}

//----------------------------------------------------------------------------

function Player::kill(%this, %damageType){
  %this.damage(0, %this.getPosition(), 10000, %damageType);
}

//----------------------------------------------------------------------------

function Player::playDeathCry( %this ){
  %this.playAudio(0,DeathCrySound);
}

//----------------------------------------------------------------------------

function Player::playPain( %this ){
  %this.playAudio(0,PainCrySound);
}

//----------------------------------------------------------------------------
// switches the current weapon
function Player::switchWeapon(%this, %wNum){
  // does any necessary tasks to REMOVE the current weapon before adding the new weapon
  switch(%this.currentWeapon){
    case 0:
      %this.unmountImage(mindWeapon);   
    case 1:
      %this.unmountImage(NeutralizerWeapon);   
  }  
  
  // now do any tasks necessary to ADD the new weapon choice
  switch(%wNum){
    case 0:
      %this.mountImage(mindWeapon, 0);   
      %this.setImageAmmo( 0, 1 );
    case 1:
      %this.mountImage(NeutralizerWeapon, 0);

      
      // give this weapon "ammo" 
      //  setImageAmmo( slot , hasAmmo ) 
      //  slot  An integer value between 0 and 7 representing a mount slot on this shape.
      //  hasAmmo  A boolean value. If set to true, this image is set has having ammo, otherwise it is set as not having ammo.
      %this.setImageAmmo( 0, 1 );

  }  
  
  // save new weapon selection as current
  %this.currentWeapon = %wNum;
}

//----------------------------------------------------------------------------





//----------------------------------------------------------------------------
// Armor methods
//----------------------------------------------------------------------------

//----------------------------------------------------------------------------

function Armor::onCollision(%this,%obj,%col){
  //if (%obj.getState() $= "Dead")
  //  return;

  // Try and pickup all items
  //if (%col.getClassName() $= "Item")
  //  %obj.pickup(%col);

  // Mount vehicles
  //%this = %col.getDataBlock();
  //if ((%this.className $= WheeledVehicleData) && %obj.mountVehicle && %obj.getState() $= "Move" && %col.mountable){
  //  // Only mount drivers for now.
  //  %node = 0;
  //  %col.mountObject(%obj,%node);
  //  %obj.mVehicle = %col;
  //}
}

//----------------------------------------------------------------------------

function Armor::onImpact(%this, %obj, %collidedObject, %vec, %vecLen){
  if(%obj.getClassName() $= "AIPlayer")
    %obj.damage(0, VectorAdd(%obj.getPosition(),%vec),%vecLen * %this.speedDamageScale, "Impact");
}

function Armor::damage(%this, %obj, %sourceObject, %position, %damage, %damageType ){
  
  // If we're already dead, don't bother with it...
  if( %obj.getState() $= "Dead" )
    return;

  %obj.applyDamage( %damage );

  // Deal with client callbacks here because we don't have this information 
  // in the onDamage or onDisabled methods
  %client = %obj.client;
  %sourceClient = %sourceObject ? %sourceObject.client : 0;

  if( %obj.getState() $= "Dead" ){
    if(!(%obj.getClassName() $= "AIPlayer"))
      %client.onDeath( %sourceObject, %sourceClient, %damageType, %location );
  }
}

function Armor::onDamage(%this, %obj, %delta){
  %obj.severeDamageFlash = 0.65;
  %obj.slightDamageFlash = 0.40;
  %obj.minDamageFlash    = 0.10;
  %client = %obj.client;
  if(%delta > 3.0){
    // This method is invoked by the ShapeBase code whenever the 
    // object's damage level changes.
    if (%obj.getState() !$= "Dead"){
      // Increment the flash based on the amount.
      %flash = %obj.getDamageFlash();// + ((%delta / %this.maxDamage) * 2);
      
      // If the pain is excessive, let's hear about it.
      if (%delta > 10){
        %obj.playPain();
        %flash = %obj.severeDamageFlash;
      }else{
        %obj.playPain();
        %flash = %obj.slightDamageFlash + %flash;
        if (%flash > %obj.severeDamageFlash){
          %flash = %obj.severeDamageFlash;  
        }
      }

      %obj.setDamageFlash(%flash);
    }
  }//end of first if
  else if (%delta > 0){
    // This method is invoked by the ShapeBase code whenever the 
    // object's damage level changes.
    if (%obj.getState() !$= "Dead") {
      // Increment the flash based on the amount.
      %flash = %obj.minDamageFlash;
      %obj.setDamageFlash(%flash);
    }
  }//end of else
}

function Armor::onDisabled( %this, %obj, %state ){
  // The player object sets the "disabled" state when damage exceeds
  // it's maxDamage value.  This is method is invoked by ShapeBase
  // state mangement code.

  %obj.setRepairRate(0.00);
  
  // If we want to deal with the damage information that actually
  // caused this death, then we would have to move this code into
  // the script "damage" method.
  %obj.playDeathCry();
  %obj.playDeathAnimation();
  %obj.setDamageFlash(0.75);

  // Release the main weapon trigger
  %obj.setImageTrigger(0,false);

  // Schedule time and duration of fade-out for dead players. 
  %obj.schedule( 6000, "startFade", 1000, 0, true );

  // Schedule deletion for dead players. 
  %obj.schedule( 8000, "delete" );
   
  // Schedule corpse removal.  Just keeping the place clean.
  //%obj.schedule($CorpseTimeoutValue - 1000, "startFade", 1000, 0, true);
  //%obj.schedule($CorpseTimeoutValue, "delete");  
}

//-----------------------------------------------------------------------------
// refills the sprayer a bit
function Player::refillSprayer(%obj){
  %obj.sprayerLevel += 10;
  if (%obj.sprayerLevel >= 100){
    %obj.schedRefill = "";
    %obj.sprayerLevel = 100;
  }else{
    %obj.schedRefill = %obj.schedule(2000, refillSprayer, %obj);
  }
}