// Storyline Handler
//
// Object is created inside of GameConnection object when the player is created
//
//
// StorylineHandler handles all things that need to happen when various events occur that could effect the current storyline
//

// just for array size
$StorylineHandler::maxNumCheckpoints = 100;
$StorylineHandler::haha =0;//Funny things


// this saves an item's properties so that if the game is restored, we can recreate the item over again if it was deleted between the save and the restore events
// %this is passed automatically by the engine
// %itemname is just any string that will identify the item so we can restore it
// %item is the ID of the object in the engine
function StorylineHandler::SaveItem(%this, %itemName, %item){
  %this.savedItemPosition[%itemname] = getWords(%item.getTransform(),0,2);
  %this.savedItemScale[%itemname] = %item.scale;
  %this.savedItemGroup[%itemname] = %item.getGroup();
  %this.savedItemName[%itemname] = %item.getName();
  %this.savedItemDatablock[%itemname] = %item.getDatablock();
}


// this recreates an item with the properties saved by SaveItem()
// %this is passed automatically by the engine
// %itemname is the itemname string that was used to identify the item when SaveItem() was called
function StorylineHandler::RestoreItem(%this, %itemName){
  %obj = %this.savedItemDatablock[%itemname].create(%this.savedItemName[%itemname]);
  %obj.setTransform(%this.savedItemPosition[%itemname] SPC "0 0 0 1");
  %obj.setScale(%this.savedItemScale[%itemname]);
  %this.savedItemGroup[%itemname].add(%obj);
}

// initializes our storyline
function StorylineHandler::initialize(%this, %ourClient){
  // all checkpoints have not yet been reached on initialize
  //for (%i = 0; %i < %this.maxNumCheckpoints; %i++){
  //  %this.checkpointReached[%i] = false;
  //}
  
  // save pointer to our client
  %this.ourClient = %ourClient;
  
  // save info about our NeutralizerItem object
  %this.SaveItem("Neutralizer", NeutralizerItem.getID());
  %this.SaveItem("Agent", AgentItem.getID());
  
  // and remember that currently, we don't have the neutralizer
  %this.gotNeutralizer = false;
  %this.gotAgent = false;
  %this.poisonedTheVat = false;
  %this.escapedTheFactory = false;
  
  //%this.objectiveIDs["Agent"]
  %this.objectiveText["Neutralizer"] = "Pick up the Neutralizer.";
  %this.objectiveText["SFact"] = "Sneak into the human factory.";
  %this.objectiveText["Agent"] = "Retrieve the Chemical Agent from its hidden location.";
  %this.objectiveText["MVat"] = "Introduce the Agent and Neutralizer into the Master Vat.";
  %this.objectiveText["Escape"] = "Escape the factory before you die from the poisonous gas.";
  %this.objectiveText["Return"] = "Return to Xent Rthur (your ally) at your starting location.";
}

// changes the color of the particles emitted by the vats 
// for use once they've been poisoned
function StorylineHandler::masterVatPoisoned(%this, %isPoisoned){
  if (%isPoisoned == true){
    
  
  // if we need to REVERT the effects
  }else{
  
  }
}

function StorylineHandler::addObjective(%this, %ob){
  %this.objectiveIDs[%ob] = ingameui.addObjective(%this.objectiveText[%ob]);
}

function StorylineHandler::completeObjective(%this, %ob){
  ingameui.completeObjective(%this.objectiveIDs[%ob]);
}

function StorylineHandler::constantDamage(%this){
  if (!isObject(%this.ourClient.player)){
    return;    
  }
  %this.ourClient.player.damage("", "0 0 0", 0.6, "Gas");
  
  if (%this.escapedTheFactory == false)
    %this.schedule(1000, constantDamage);
}

function StorylineHandler::constantDamageAI(%this){
  // Damage Soldiers
  InitContainerRadiusSearch("0 0 0", 8000.0, $TypeMasks::PlayerObjectType);
  while ((%targetObject = containerSearchNext()) != 0)
  	 if(%targetobject.getclassname() $= "AIPlayer")
  	   if (%targetobject.getDataBlock().getName() $= "SoldierPrivate")
  		    %targetobject.Damage("", "0 0 0", 1.4, "Gas");
  
  %this.schedule(800, constantDamageAI);
}



// handles player to be stationary
function StorylineHandler::StationaryPlayer(%this){
  %db = %this.ourClient.player.getDatablock();
  %db.maxForwardSpeed = 14;
  %db.maxBackwardSpeed = 13;
  %db.maxSideSpeed = 13;
}

function StorylineHandler::introSpeak(%this, %message, %time){
  bottomPrintAll(%message, %time, 10);
  AllyPlayer.playAudio(2, AllyPlayerSpeakSound);
}

// handles storyline events that occur
function StorylineHandler::signal(%this, %message, %arg1, %arg2){
  // lower-case the message string
  %message = strlwr(%message);


 
  // if the game has been saved,
  if (%message $= "save"){
  
    // save state of whether we have neutralizer
    %this.gotNeutralizerSaved = %this.gotNeutralizer;
    %this.gotAgentSaved = %this.gotAgent;
    %this.poisonedTheVatSaved = %this.poisonedTheVat;
    %this.escapedTheFactorySaved = %this.escapedTheFactory;
  
  // endif game has been saved  
    
  
  
  // if the game has been restored  
  }else if (%message $= "restore"){
    %this.haha = 0;
    
    // if in our previous state, we didn't have the neutralizer, but now we do, we need to revert
    if (%this.gotNeutralizerSaved != %this.gotNeutralizer){
      %this.gotNeutralizer = false;

      %this.restoreItem("Neutralizer");
    }
    
    // if in our previous state, we didn't have the agent, but now we do, we need to revert
    if (%this.gotAgentSaved != %this.gotAgent){
      %this.gotAgent = false;

      %this.restoreItem("Agent");
    }
    
    if (%this.poisonedTheVatSaved != %this.poisonedTheVat){
      %this.poisonedTheVat = false;
      %this.masterVatPoisoned(false);
      
      updateEffects(false);
    }
    
    if (%this.escapedTheFactorySaved != %this.escapedTheFactory){
      %this.escapedTheFactory = false;
    }
    
    if ((%this.escapedTheFactory == false) && (%this.poisonedTheVat == true)){
      // damage the player constantly
      %this.ourClient.player.setRepairRate(0.0);
      %this.constantDamage();
      %this.constantDamageAI();
    }else{
      %this.ourClient.player.setRepairRate(%this.ourClient.player.ourConstantRepairRate);
    }
  
    AIManager.createSoldiers(%arg1); 
  // endif game has been restored 
    
   

  // if we reached a checkpoint   
  }else if (%message $= "reachcheckpoint"){
    if ((%arg1 != 0) && (%arg1 != 4)){
      bottomPrintAll("\c4Checkpoint Reached!", 6, 3);
    } 
    
    switch(%arg1){
      // we are at our starting location
      case 0:
        %this.ourClient.setFirstPerson(false);
        
        %db = %this.ourClient.player.getDatablock();
        %db.maxForwardSpeed = 0;
        %db.maxBackwardSpeed = 0;
        %db.maxSideSpeed = 0;

        %this.schedule(5000, introSpeak, "\c4Attention Xophad! You are the strongest", 5);
        %this.schedule(10000, introSpeak, "\c4and the most crazy bugger in the galaxy!", 5, 10);
        %this.schedule(15000, introSpeak, "\c4We need you to destroy this human outpost", 5, 10);
        %this.schedule(20000, introSpeak, "\c4with your swift bug reflexes.", 5, 10);
        %this.schedule(25000, introSpeak, "\c4Your Mission: 1)Infiltrate the Outpost.", 5, 10);
        %this.schedule(30000, introSpeak, "\c42)Find the Chemical Agent.", 5, 10);
        %this.schedule(35000, introSpeak, "\c43)Sabotage the Master Vat of insecticide.", 5, 10);
        %this.schedule(40000, introSpeak, "\c44)Escape the factory and return here.", 5, 10);
        %this.schedule(45000, introSpeak, "\c4Help us, Xophad, you're our only hope!", 5, 10);
        %this.schedule(50000, introSpeak, "\c4Don't forget the neutralizer over there.", 5, 10);
        
        // unfreeze player
        %this.schedule(50000, StationaryPlayer);
        
        // move ally player
        %db = AllyPlayer.getDatablock();
        %db.schedule(53000,moveToFinalLocation,AllyPlayer.getId());
     
        
        %this.addObjective("Neutralizer");
        %this.addObjective("SFact");
      
      case 1:
        %this.completeObjective("SFact");
        %this.addObjective("Agent");
        %this.addObjective("MVat");
        schedule(6000, 0, bottomPrintAll, "\c4Now, find the hidden Chemical Agent!", 7, 3);
        
    }
    
    
  // endif we reached a checkpoint
  
  
  
  // player has picked up the neutralizer item
  }else if (%message $= "getNeutralizer"){
    
    // make the objective in the list checked
    %this.completeObjective("Neutralizer");
      
    // remember that player picked it up
    %this.gotNeutralizer = true;
    
    bottomPrintAll("\c4Picked up the neutralizer and a sprayer!\n\c4Press 2 to select the sprayer.", 7, 3);

  // endif player got the neutralizer item
  
  
  
  // if player has picked up the agent  
  }else if (%message $= "getAgent"){
    %this.completeObjective("Agent");
    
    %this.gotAgent = true;
         
    bottomPrintAll("\c4Found the Agent!", 7, 3);
  
  
  
  // if we've reached the master vat
  }else if(%message $= "reachMasterVat"){
    if (%this.poisonedTheVat == true) return;
    
    // if user has the neutralizer
    if (%this.ourClient.inventoryContains("Neutralizer") == true){
      // if user has the agent
      if (%this.ourClient.inventoryContains("Agent") == true){
        %this.poisonedTheVat = true;
        %this.masterVatPoisoned(true);
        %this.completeObjective("MVat");
        %this.addObjective("Escape");
        alxPlay(PoisonGasParticleEmitter.soundProfile);//play poison gas sound
        bottomPrintAll("\c4You introduced the neutralizer and\n\c4agent into the master vat!", 6, 3);
        updateEffects(true);
        schedule(3000,0, createPoisonGasEmitters);
        schedule(6000,0, bottomPrintAll, "\c4Quickly! Escape from the factory\n\c4before you die from the gas!", 10, 3);
        
        // ok, save this place as the last checkpoint now
        %this.ourClient.lastCheckpoint = 4;
        %this.ourClient.storylineHandler.signal("reachcheckpoint",4);   
        %this.ourClient.saveGameState();
        
        %this.ourClient.player.setRepairRate(0.0);
        %this.constantDamage();
        %this.constantDamageAI();
            
        echo("Player switched checkpoints.  Now respawning at #4.  (issued by storylineHandler.cs)");
      
      // if user does not have the agent
      }else{
        bottomPrintAll("\c4You can't poison the master vat\n\c4without the chemical agent!", 6, 3);  
      }
      
      
    // if user does not have neutralizer     
    }else{
      bottomPrintAll("\c4You can't poison the master vat\n\c4without the neutralizer!", 6, 3);
    }
  
  
  
  // if we've reached our ally at the end of the game
  }else if(%message $= "returnToAlly"){
    if (%this.poisonedTheVat == true){
      %this.completeObjective("Return");
      //show congrates text
      if(%this.haha==0)
      {
      %this.schedule(500, introSpeak, "\c4Xophad you are the hero!", 5);
      %this.schedule(3000, introSpeak, "\c4  Congratulations, You Beat the Game!  ", 5, 10);
      %this.haha++;
      }
      else if(%this.haha==1)
      {
      %this.schedule(500, introSpeak, "\c4Stop Talking to me...you beat the game!", 5);
      %this.schedule(4000, introSpeak, "\c4  Just try talking to me again!  ", 5, 10);
      %this.haha++;
      }
      else if(%this.haha==2)
      {
      %this.schedule(500, introSpeak, "\c4 Take this...Bam!!! Muhahaha!", 5);
      %this.ourClient.player.setTransform("-42.0818 312.094 133.736");
      %this.haha=0;
      }
      %this.schedule(6000,GoToMain);//bring up menu exit screen
    }
  }
  
}

function StorylineHandler::GetPoisonState(%this){
  return %this.poisonedTheVat;
}

function StorylineHandler::GoToMain(%this)
{
   ingameUI.exitToMainMenu_Click();
}




function skipIntro(){
  $MainPlayer.client.storylineHandler.StationaryPlayer();
}

function StorylineHandler::stopSound(%this, %obj){
  %obj.stopAudio(0);
}